@@ -1,5 +1,6 @@
 """Executable of fabsetup"""
 
+import collections
 import datetime
 import os
 import os.path
@@ -9,11 +10,13 @@ import sys
 
 # import fabric
 import fabric.config
+
 # import fabric.connection
 import fabric.main
 import invoke
 import invoke.main
 import invoke.config
+
 # import invoke.context
 import invoke.parser
 from invoke.util import debug
@@ -22,9 +25,333 @@ import fabsetup.addons
 import fabsetup.print
 import fabsetup.utils.outfile
 import fabsetup.utils.pandoc
+
 # import fabsetup.fabutils.queries
 
 
+class Defaults:
+    """Fabsetup defaults."""
+
+    def __init__(self):
+
+        Entry = collections.namedtuple(
+            typename="Default",
+            field_names=["key", "default_value", "description"],
+        )
+
+        self.entries = [
+            Entry(
+                "outfile",
+                description="Configure if and how an outfile will be created.",
+                default_value=[
+                    Entry(
+                        "name",
+                        "",
+                        "Path of the outfile where the markdown output will be "
+                        "(over-) written to. If the value is empty on task "
+                        "execution, no outfile will be created.",
+                    ),
+                    Entry(
+                        "dir",
+                        "",
+                        "If non-empty and ``outfile.name`` is empty set the "
+                        "value of ``outfile.name`` to "
+                        '``f"{outfile.dir}/{basename}"`` '
+                        "before task execution. ``basename`` is the "
+                        "formatted ``outfile.basename_formatter``.",
+                    ),
+                    Entry(
+                        "basename_formatter",
+                        "fabsetup_{now}{tasks}{hosts}.md",
+                        "Create a variable filename, by default containing the "
+                        "date and time of execution, the names of the exectued "
+                        "tasks and the name of the hosts (if any was given by "
+                        "``-H``), each part prepended by an underscore. "
+                        "The default value results in a filename like "
+                        "``'fabsetup_2021-02-21_12:30:45_task1_task2_host1_host2.md'``"
+                        " for example.",
+                    ),
+                    Entry(
+                        "now_format",
+                        "%F_%H-%M-%S",
+                        "the ``now`` variable in ``outfile.basename_formatter`` "
+                        "is set with the current date: "
+                        "``now = datetime.datetime.now().strftime(now_format)``.",
+                    ),
+                    Entry(
+                        "keep_color",
+                        False,
+                        "When ``True`` do not remove ANSI color codes from the "
+                        "markdown outfile.",
+                    ),
+                    Entry(
+                        "pandoc",
+                        description="Configure file modifications and "
+                        "conversions applied by pandoc.",
+                        default_value=[
+                            Entry(
+                                "command",
+                                "pandoc",
+                                "Pandoc executable, could be set for example to "
+                                "``'pandoc'`` or ``'/usr/bin/pandoc'``.",
+                            ),
+                            Entry(
+                                "toc",
+                                False,
+                                "If ``True`` add a table of contents to the "
+                                "markdown outfile.",
+                            ),
+                            Entry(
+                                "html",
+                                description="Configure HTML file creation.",
+                                default_value=[
+                                    Entry(
+                                        "name",
+                                        "",
+                                        "If non-empty on task execution and a "
+                                        "markdown outfile exists use pandoc to "
+                                        "convert the markdown to an HTML file.",
+                                    ),
+                                    Entry(
+                                        "dir",
+                                        "",
+                                        "If non-empty and "
+                                        "``outfile.pandoc.html.name`` is empty "
+                                        "set ``outfile.pandoc.html.name`` to "
+                                        '``f"{dir}/{basename}.html"`` '
+                                        "before task execution. ``basename`` is "
+                                        "the basename of the markdown outfile "
+                                        "without the trailing ``.md``.",
+                                    ),
+                                    Entry(
+                                        "css",
+                                        [
+                                            Entry(
+                                                "disabled",
+                                                False,
+                                                "If ``True`` the created HTML "
+                                                "file will not use CSS.",
+                                            ),
+                                            Entry(
+                                                "inline",
+                                                True,
+                                                "If ``True`` embed the CSS "
+                                                "inline into the HTML file, "
+                                                "else create a CSS file next "
+                                                "to it.",
+                                            ),
+                                            Entry(
+                                                "url",
+                                                "",
+                                                "",
+                                            ),
+                                            Entry(
+                                                "auto_remove_markdown_file",
+                                                True,
+                                                "",
+                                            ),
+                                        ],
+                                        "Configure Cascading Style Sheets.",
+                                    ),
+                                ],
+                            ),
+                        ],
+                    ),
+                    Entry(
+                        "prepend_executed_fabsetup_command",
+                        True,
+                        "",
+                    ),
+                    Entry(
+                        "fabsetup_command_prefix",
+                        "*Executed fabsetup command:*\n\n",
+                        "",
+                    ),
+                    Entry(
+                        "command_output_prefix",
+                        "(stdout) ",
+                        "",
+                    ),
+                    Entry(
+                        "command_errput_prefix",
+                        "(STDERR) ",
+                        "",
+                    ),
+                ],
+            ),
+            Entry(
+                "output",
+                description="Configure the produced output on fabsetup task execution.",
+                default_value=[
+                    Entry(
+                        "color",
+                        [
+                            Entry(
+                                "cmd_local",
+                                "green",
+                                "",
+                            ),
+                            Entry(
+                                "cmd_remote",
+                                "yellow",
+                                "",
+                            ),
+                            Entry(
+                                "docstring",
+                                "blue",
+                                "",
+                            ),
+                            Entry(
+                                "error",
+                                "red",
+                                "",
+                            ),
+                            Entry(
+                                "full_name",
+                                "no_color",
+                                "",
+                            ),
+                            Entry(
+                                "subtask_heading",
+                                "cyan",
+                                "",
+                            ),
+                            Entry(
+                                "task_heading",
+                                "magenta",
+                                "",
+                            ),
+                            # Entry("question", "default_color", "",),
+                        ],
+                        "color configuration",
+                    ),
+                    Entry(
+                        "color_off",
+                        False,
+                        "",
+                    ),
+                    Entry(
+                        "hide_command_line",
+                        False,
+                        "",
+                    ),
+                    Entry(
+                        "hide_code_block",
+                        False,
+                        "",
+                    ),
+                    Entry(
+                        "hide_docstring",
+                        False,
+                        "",
+                    ),
+                    Entry(
+                        "hide_heading",
+                        False,
+                        "",
+                    ),
+                    Entry(
+                        "hide_print",
+                        False,
+                        "",
+                    ),
+                    Entry(
+                        "numbered",
+                        True,
+                        "",
+                    ),
+                    Entry(
+                        "numbered_state",
+                        "0",
+                        "",
+                    ),
+                    Entry(
+                        "task_depth",
+                        1,
+                        "",
+                    ),
+                ],
+            ),
+            Entry(
+                "run",
+                description="run configuration",
+                default_value=[
+                    Entry(
+                        "interactive",
+                        False,
+                        "",
+                    ),
+                ],
+            ),
+            Entry(
+                "load_invoke_tasks_file",
+                False,
+                "",
+            ),
+            Entry(
+                "load_fabric_fabfile",
+                False,
+                "",
+            ),
+            # "search_root": None,
+            Entry(
+                "run_before",
+                "",
+                "Command hook to be executed before fabsetup execution.",
+            ),
+            Entry(
+                "run_finally",
+                "",
+                "Command hook to be executed after fabsetup execution.",
+            ),
+        ]
+
+    @staticmethod
+    def _as_dict_r(entries):
+        if type(entries) is list:
+            res = {}
+            for entry in entries:
+                res[entry.key] = Defaults._as_dict_r(entry.default_value)
+            return res
+        else:
+            default_value = entries
+            return default_value
+
+    def as_dict(self):
+        """Return this ``fabsetup.main.Defaults`` as dict."""
+        return Defaults._as_dict_r(self.entries)
+
+    @staticmethod
+    def _as_restructuredtext_items_r(entries, depth=0):
+        if type(entries) is list:
+            for entry in entries:
+                print("")
+                # print("  " * depth + f"* ``{entry.key}``: {entry.description}")
+                print(
+                    "  " * depth
+                    + "* ``{entry.key}``: {entry.description}".format(entry=entry)
+                )
+                Defaults._as_restructuredtext_items_r(
+                    entry.default_value, depth=depth + 1
+                )
+        else:
+            default_value = entries
+            if default_value == "":
+                print("  " * depth + "Empty string as default.")
+            else:
+                # print("  " * depth + f"Default value is ``{repr(default_value)}``.")
+                print(
+                    "  " * depth
+                    + "Default value is ``{default_value}``.".format(
+                        default_value=repr(default_value)
+                    )
+                )
+
+    def as_restructuredtext_items(self):
+        """Print configuration as list items in restructuredtext."""
+        Defaults._as_restructuredtext_items_r(self.entries)
+
+
 class FabsetupConfig(fabric.config.Config):
     """A `fabric.config.Config` subclass which is an `invoke.config.Config`
     subclass for prefix manipulation.
@@ -74,62 +401,7 @@ class FabsetupConfig(fabric.config.Config):
             `dict` with merged defaults.
         """
         defaults = fabric.config.Config.global_defaults()
-        ours = {
-            "outfile": {
-                "dir": "",
-                "basename_formatter": "fabsetup_{now}{tasks}{hosts}.md",
-                "now_format": "%F_%H-%M-%S",
-                "name": "",
-                "keep_color": False,
-                "pandoc": {
-                    "command": "pandoc",
-                    "html": {
-                        "dir": "",
-                        "name": "",
-                        "css": {
-                            "disabled": False,
-                            "inline": True,
-                            "url": "",
-                            "auto_remove_markdown_file": True,
-                        },
-                    },
-                    "toc": False,
-                },
-                "prepend_executed_fabsetup_command": True,
-                "fabsetup_command_prefix": "*Executed fabsetup command:*\n\n",
-                "command_output_prefix": "(stdout) ",
-                "command_errput_prefix": "(STDERR) ",
-            },
-            "output": {
-                "color": {
-                    "cmd_local": "green",
-                    "cmd_remote": "yellow",
-                    "docstring": "blue",
-                    "error": "red",
-                    "full_name": "no_color",
-                    "subtask_heading": "cyan",
-                    "task_heading": "magenta",
-                    # "question": "default_color",
-                },
-                "color_off": False,
-                "hide_command_line": False,
-                "hide_code_block": False,
-                "hide_docstring": False,
-                "hide_heading": False,
-                "hide_print": False,
-                "numbered": True,
-                "numbered_state": "0",
-                "task_depth": 1,
-            },
-            "run": {
-                "interactive": False,
-            },
-            "load_invoke_tasks_file": False,
-            "load_fabric_fabfile": False,
-            # "search_root": None,
-            "run_before": "",
-            "run_finally": "",
-        }
+        ours = Defaults().as_dict()
         invoke.config.merge_dicts(defaults, ours)
         return defaults
 
